"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const http_1 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const disposable_1 = require("@reactive-js/disposable");
const option_1 = require("@reactive-js/option");
class RequestBody extends disposable_1.AbstractDisposable {
    constructor(req) {
        super();
        this.req = req;
        this.consumed = false;
        this.add(_ => {
            req.removeAllListeners();
            req.once("error", () => { });
            req.once("close", () => {
                req.removeAllListeners();
            });
            req.destroy();
        });
        const onError = (cause) => {
            this.dispose({ cause });
        };
        req.on("error", onError);
    }
    enumerateAsync(scheduler, replayCount) {
        if (this.consumed) {
            throw new Error("Request body already consumed");
        }
        this.consumed = true;
        const sink = node_1.createBufferStreamFromReadable(() => this.req)
            .enumerateAsync(scheduler, replayCount)
            .add(this);
        this.add(sink);
        return sink;
    }
}
class ResponseBody extends disposable_1.AbstractDisposable {
    constructor(resp) {
        super();
        this.resp = resp;
        this.consumed = false;
        this.add(_ => {
            resp.removeAllListeners();
            resp.once("error", () => { });
            resp.once("close", () => {
                resp.removeAllListeners();
            });
            resp.destroy();
        });
        const onError = (cause) => {
            this.dispose({ cause });
        };
        resp.on("error", onError);
    }
    enumerateAsync(scheduler, replayCount) {
        if (this.consumed) {
            throw new Error("Response body already consumed");
        }
        this.consumed = true;
        const sink = node_1.createBufferStreamSinkFromWritable(() => this.resp, true)
            .enumerateAsync(scheduler, replayCount)
            .add(this);
        this.add(sink);
        return sink;
    }
}
const writeResponseMessage = (responseBody) => (response) => {
    responseBody.resp.statusCode = response.statusCode;
    http_1.writeHttpResponseHeaders(response, (header, value) => responseBody.resp.setHeader(header, value));
};
const writeResponseBody = (responseBody) => ({ body, contentInfo, }) => option_1.isSome(contentInfo) ? async_enumerable_1.sink(body, responseBody) : observable_1.empty();
const defaultOnError = (_) => observable_1.empty();
exports.createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (requestBody, responseBody) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = requestBody.req;
        const isTransportSecure = (_a = requestBody.req.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        return pipe_1.pipe(() => http_1.parseHttpRequestFromHeaders({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body: requestBody,
        }), observable_1.compute, observable_1.await_(handler), observable_1.onNotify(writeResponseMessage(responseBody)), observable_1.await_(writeResponseBody(responseBody)), observable_1.catchError(onError));
    };
    return (req, resp) => {
        const requestBody = new RequestBody(req);
        const responseBody = new ResponseBody(resp);
        responseBody.add(pipe_1.pipe(handleRequest(requestBody, responseBody), observable_1.subscribe(scheduler)).add(requestBody));
    };
};
//# sourceMappingURL=httpRequestListener.js.map