"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const mime_db_1 = tslib_1.__importDefault(require("mime-db"));
const http_1 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const option_1 = require("@reactive-js/option");
const pipe_1 = require("@reactive-js/pipe");
const httpContentEncoding_1 = require("./httpContentEncoding");
const httpMessage_1 = require("./httpMessage");
exports.decodeHttpResponse = (options) => response => httpMessage_1.decodeHttpMessage(response, options);
exports.encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = tslib_1.__rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = option_1.isSome(shouldEncodeOption)
        ? shouldEncodeOption(request, response)
        : option_1.none;
    const shouldEncode = option_1.isSome(shouldEncodeOptionResult)
        ? shouldEncodeOptionResult
        : http_1.httpResponseIsCompressible(response, mime_db_1.default);
    const { preferences } = request;
    const acceptedEncodings = option_1.isSome(preferences) && shouldEncode ? preferences.acceptedEncodings : [];
    const { body, contentInfo, vary } = response;
    const encoding = httpContentEncoding_1.getFirstSupportedEncoding(acceptedEncodings !== null && acceptedEncodings !== void 0 ? acceptedEncodings : []);
    return option_1.isSome(encoding) &&
        option_1.isSome(contentInfo) &&
        contentInfo.contentEncodings.length === 0
        ? Object.assign(Object.assign({}, response), { body: pipe_1.pipe(body, node_1.transform(httpContentEncoding_1.createEncodingCompressTransform(encoding, zlibOptions))), contentInfo: Object.assign(Object.assign({}, contentInfo), { contentEncodings: [encoding], contentLength: -1 }), vary: [...vary, "Accept-Encoding"] }) : response;
};
exports.encodeCharsetHttpResponse = (contentType) => {
    const messageEncoder = httpMessage_1.encodeCharsetHttpMessage(contentType);
    return resp => messageEncoder(resp);
};
//# sourceMappingURL=httpResponse.js.map