import { __rest } from "tslib";
import iconv from "iconv-lite";
import { parseMediaTypeOrThrow } from "@reactive-js/http";
import { transform } from "@reactive-js/node";
import { isSome } from "@reactive-js/option";
import { pipe } from "@reactive-js/pipe";
import { createEncodingDecompressTransform, } from "./httpContentEncoding";
import { ofValueStream } from "@reactive-js/async-enumerable";
export const decodeHttpMessage = (message, options = {}) => {
    const { body, contentInfo } = message;
    return isSome(contentInfo) && contentInfo.contentEncodings.length > 0
        ? Object.assign(Object.assign({}, message), { body: contentInfo.contentEncodings
                .map(encoding => createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe(acc, transform(decoder)), body), content: Object.assign(Object.assign({}, contentInfo), { contentEncodings: [], contentLength: -1 }) }) : message;
};
export const encodeCharsetHttpMessage = (contentType) => {
    var _a;
    const parsedContentType = typeof contentType === "string"
        ? parseMediaTypeOrThrow(contentType)
        : contentType;
    const charset = (_a = parsedContentType.params["charset"]) !== null && _a !== void 0 ? _a : "utf-8";
    return response => {
        const { body } = response, responseWithoutBody = __rest(response, ["body"]);
        const buffer = iconv.encode(response.body, charset);
        const streamBody = ofValueStream(buffer);
        return Object.assign(Object.assign({}, responseWithoutBody), { body: streamBody, contentInfo: {
                contentType: parsedContentType,
                contentLength: buffer.length,
                contentEncodings: [],
            } });
    };
};
//# sourceMappingURL=httpMessage.js.map