import { __rest } from "tslib";
import db from "mime-db";
import { httpResponseIsCompressible, } from "@reactive-js/http";
import { transform } from "@reactive-js/node";
import { isSome, none } from "@reactive-js/option";
import { pipe } from "@reactive-js/pipe";
import { getFirstSupportedEncoding, createEncodingCompressTransform, } from "./httpContentEncoding";
import { decodeHttpMessage, encodeCharsetHttpMessage } from "./httpMessage";
export const decodeHttpResponse = (options) => response => decodeHttpMessage(response, options);
export const encodeHttpResponse = (request, options = {}) => response => {
    const { shouldEncode: shouldEncodeOption } = options, zlibOptions = __rest(options, ["shouldEncode"]);
    const shouldEncodeOptionResult = isSome(shouldEncodeOption)
        ? shouldEncodeOption(request, response)
        : none;
    const shouldEncode = isSome(shouldEncodeOptionResult)
        ? shouldEncodeOptionResult
        : httpResponseIsCompressible(response, db);
    const { preferences } = request;
    const acceptedEncodings = isSome(preferences) && shouldEncode ? preferences.acceptedEncodings : [];
    const { body, contentInfo, vary } = response;
    const encoding = getFirstSupportedEncoding(acceptedEncodings !== null && acceptedEncodings !== void 0 ? acceptedEncodings : []);
    return isSome(encoding) &&
        isSome(contentInfo) &&
        contentInfo.contentEncodings.length === 0
        ? Object.assign(Object.assign({}, response), { body: pipe(body, transform(createEncodingCompressTransform(encoding, zlibOptions))), contentInfo: Object.assign(Object.assign({}, contentInfo), { contentEncodings: [encoding], contentLength: -1 }), vary: [...vary, "Accept-Encoding"] }) : response;
};
export const encodeCharsetHttpResponse = (contentType) => {
    const messageEncoder = encodeCharsetHttpMessage(contentType);
    return resp => messageEncoder(resp);
};
//# sourceMappingURL=httpResponse.js.map